// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KWIDGETSADDONS_VERSION_H
#define KWIDGETSADDONS_VERSION_H

#define KWIDGETSADDONS_VERSION_STRING "5.39.0"
#define KWIDGETSADDONS_VERSION_MAJOR 5
#define KWIDGETSADDONS_VERSION_MINOR 39
#define KWIDGETSADDONS_VERSION_PATCH 0
#define KWIDGETSADDONS_VERSION ((5<<16)|(39<<8)|(0))

#endif
